function [forecastData]=RandomTree(forecastData)
%
% Description: Perform logistic regression on a forecast dataset
%
nDataSet=length(forecastData.ForecastVar.Attrib);
%
% Verify that all variables are real.
%
forecastBinomial=1;
for k=1:nDataSet
    if ~strcmpi(forecastData.ForecastVar.Attrib(k).DataType,'binary')
        vFields=[];
        if ~isempty(forecastData.ForecastVar.Attrib(k).DataFieldList)
            vFields=fieldnames(forecastData.ForecastVar.Attrib(k).DataFieldList);
        end
        if ~isempty(vFields)
            forecastBinomial=0;
        end
    end
end
dataReal=1;
for k=1:nDataSet
    if ~strcmpi(forecastData.DataVar.Attrib(k).DataType,'real')
        vFields=[];
        if ~isempty(forecastData.DataVar.Attrib(k).DataFieldList)
            vFields=fieldnames(forecastData.DataVar.Attrib(k).DataFieldList);
        end
        if ~isempty(vFields)
            dataReal=0;
        end
    end
end
if (forecastBinomial==0) || (dataReal==0)
    disp(['RandomTree: Forecast and explanatory variables must be real valued for multilinear regression']);
    return;
end
forecast=forecastData.ForecastVar.TrainingArray';
forecast(forecast~=0)=1;
rng(1); % For reproducibility
VarNames_0={forecastData.DataVar.VarLabels(:).VariableName};
VarNames_1={forecastData.ForecastVar.VarLabels(:).VariableName};
VarNames={VarNames_0{:},VarNames_1{:}};
forecastData.Forecast = fitctree(forecastData.DataVar.TrainingArray',forecast,...
    'VarNames',VarNames,'MaxNumSplits',7,'CrossVal','on');
%forecastData.Forecast = fitctree(forecastData.DataVar.TrainingArray',forecast,'MaxNumSplits',7);
view(forecastData.Forecast.Trained{1},'Mode','graph')
return;
end