function SWFT_v5
%
% Description: Space Weather Forecast Testbed is an interactive tool for
%       developing models for forecasting space weather conditions using 
%       machine-learning techniques based on a historical database.
% Usage: SWFT
%
% Authors: Zhujun Shen, Zongyan Jiang, Chunming Wang
% Creation date: May 30, 2019
%
global SWFTData_Path
%
% Load the default parameters for forecast experiment
%
DefaulData_filepath = which('Default_SWFT_ForecastData.mat');
if isempty(DefaulData_filepath)
    error('SWFT: Cannot find default dataset.');
end
ExistingFig=get(0,'Children');
if ~isempty(ExistingFig)
    if ~isempty(find(strcmp({ExistingFig(:).Name},'SWFT')==1))
        error('SWFT: A windows for SWFT is already open. Please close that window before starting another session.');
    end
end
%
default_data = load(DefaulData_filepath,'forecastData');
forecastData = default_data.forecastData;
clear('default_data');

%  Create the SWFT main window.
SWFT_Main = figure('NumberTitle','off','Unit','normalized','Position',...
    [0.1,0.05,0.7,0.9],'MenuBar','none','Name', 'SWFT','Visible','on');

% Create layout of main panels
File_Management = uipanel('Title','Forecast Dateset Management',...
    'FontSize',12,'Position',[.025,0.93,0.95,0.055]);
File_Management_GHandle = create_file_panel(File_Management);
%         
Time_Scale_Input = uipanel('Title','Time Parameters for Forecast Experiment'...
    ,'FontSize',12,'Position',[.025,0.615,0.95,0.3]);
Time_Scale_Input_GHandle = creat_timescale_panel(Time_Scale_Input);
%
Variable_Selection = uipanel('Title',...
    'Selection of Forecast and Covariate Variables',...
    'FontSize',12,'Position',[.025,0.155,0.95,0.445]);
Variable_Selection_GHandle = creat_variable_select_panel(Variable_Selection);
%
Action_Panel = uipanel('Title','Traing & Validation Steps','FontSize',12,...
    'Position',[.025,0.025,0.95,0.115]);
Action_Panel_GHandle = create_action_panel(Action_Panel);
return;
%
% Create file management panel and its functions.
%
    function [Panel_GHandle] = create_file_panel(Panel_Handle,varargin)
            if nargin == 1
                Panel_GHandle = struct('Panel',Panel_Handle);
                % Push buttons
                Load_pushbutton = uicontrol(Panel_Handle,'Style','pushbutton',...
                    'String','Load','Unit','normalized','Position',[0.02,0.1,0.05,0.8],...
                    'Callback',@load_pushbutton_Callback);
                Panel_GHandle.Load_pushbutton = Load_pushbutton;

                Save_pushbutton = uicontrol(Panel_Handle,'Style','pushbutton',...
                    'string','Save','Unit','normalized','Position',[0.1,0.1,0.05,0.8],...
                    'Callback',@save_pushbutton_Callback);
                Panel_GHandle.Save_pushbutton = Save_pushbutton;

                Exit_pushbutton = uicontrol(Panel_Handle,'Style','pushbutton',...
                    'string','Exit','Unit','normalized','Position',[0.93,0.1,0.05,0.8],...
                    'Callback',@exit_pushbutton_Callback);
                Panel_GHandle.Exit_pushbutton = Exit_pushbutton;
            else

            end
            return;
            %
            function load_pushbutton_Callback(source,eventdata)
                if isfield(Panel_GHandle,'Default_Input_Dir')
                    if isfield(Panel_GHandle,'Default_Filename')
                        [FileName,PathName,FilterIndex] = ...
                            uigetfile([Panel_GHandle.Default_Input_Dir,filesep,Panel_GHandle.Default_Filename]);
                    else
                        [FileName,PathName,FilterIndex] = ...
                            uigetfile([Panel_GHandle.Default_Input_Dir,filesep,'*.mat']);
                    end
                elseif isfield(Panel_GHandle,'Default_Output_Dir')
                    if isfield(Panel_GHandle,'Default_Filename')
                        [FileName,PathName,FilterIndex] = ...
                            uigetfile([Panel_GHandle.Default_Output_Dir,filesep,Panel_GHandle.Default_Filename]);
                    else
                        [FileName,PathName,FilterIndex] = ...
                            uigetfile([Panel_GHandle.Default_Output_Dir,filesep,'*.mat']);
                    end
                else
                    [FileName,PathName,FilterIndex] = uigetfile('*.mat');
                end
                if isnumeric(FileName)
                    % User canceled operation.
                    return;
                else
                    Panel_GHandle.Default_Input_Dir=PathName;
                    Panel_GHandle.Default_Filename=FileName;
                end
                new_data = load([PathName,filesep,FileName],'forecastData');
                %
                % Call update forecast data to make older version of
                % forecast dataset compatible with the new data structure.
                %
                [forecastData]=UpdateForecastData(new_data.forecastData,SWFTData_Path);
                Time_Scale_Input_GHandle = creat_timescale_panel(...
                    Time_Scale_Input,Time_Scale_Input_GHandle);
                Variable_Selection_GHandle = creat_variable_select_panel(...
                    Variable_Selection,Variable_Selection_GHandle);
                return
            end
            %
            function save_pushbutton_Callback(source,eventdata)
                [forecastData]=UpdateForecastData(forecastData,SWFTData_Path);
                if isfield(Panel_GHandle,'Default_Output_Dir')
                    if isfield(Panel_GHandle,'Default_Filename')
                        [FileName,PathName,FilterIndex] = ...
                            uiputfile([Panel_GHandle.Default_Output_Dir,filesep,Panel_GHandle.Default_Filename]);
                    else
                        [FileName,PathName,FilterIndex] = ...
                            uiputfile([Panel_GHandle.Default_Output_Dir,filesep,'*.mat']);
                    end
                elseif isfield(Panel_GHandle,'Default_Input_Dir')
                    if isfield(Panel_GHandle,'Default_Filename')
                        [FileName,PathName,FilterIndex] = ...
                            uiputfile([Panel_GHandle.Default_Input_Dir,filesep,Panel_GHandle.Default_Filename]);
                    else
                        [FileName,PathName,FilterIndex] = ...
                            uiputfile([Panel_GHandle.Default_Input_Dir,filesep,'*.mat']);
                    end
                else
                    [FileName,PathName,FilterIndex] = uiputfile('*.mat');
                end
                if isnumeric(FileName)
                    return;
                else
                    save([PathName,filesep,FileName],'forecastData');
                    Panel_GHandle.Default_Output_Dir=PathName;
                    Panel_GHandle.Default_Filename=FileName;
                    return
                end
            end
            %
            function exit_pushbutton_Callback(source,eventdata)
                delete(SWFT_Main);
                return
            end
        end
%
% 
% End of creating file management panel and its functions.
%
% Create time scale panel and its sub-functions.
%
    function [Panel_GHandle]=creat_timescale_panel(Panel_Handle,varargin)
        [c_year,c_month,c_day]=datevec(forecastData.CurrentEpoc);
        SWFT_Configuration = struct('latency',forecastData.DataLatency,...
            'lead',forecastData.ForecastPeriod,...
            'training',forecastData.nTrainingDay,...
            'validation',forecastData.nValidationDay,...
            'year',c_year,'month',c_month,'day',c_day,...
            'current_epoch',forecastData.CurrentEpoc);
        if nargin==1
            Panel_GHandle=struct('Panel',Panel_Handle);
            % Find the range of Data
            AchiveTime = getSWFT_AchiveTime(SWFTData_Path);
            % Create time scale selection panel
            Time_Scale_Plot = axes(Panel_Handle,'Position',[0.375,0.3,0.6,0.65]);
            Panel_GHandle.Time_Scale_Plot=Time_Scale_Plot;
            % Initial plot
            plotarea = plot_timescale(Time_Scale_Plot,SWFT_Configuration.current_epoch,...
                SWFT_Configuration.latency,SWFT_Configuration.lead,...
                SWFT_Configuration.training,SWFT_Configuration.validation);
            Panel_GHandle.plotarea=plotarea;
            % Label for current epoch
            lcurrent_epoch = uicontrol(Panel_Handle,'Style','text',...
                'string','Current Epoch:',...
                'Unit','normalized','Position',[0.02,0.85,0.15,0.1],...
                'HorizontalAlignment','left',...
                'FontWeight','bold','FontSize',10);

            rcurrent_epoch = uicontrol(Panel_Handle,'Style','text',...
                'String','Month/Day/Year',...
                'Unit','normalized','Position',[0.02,0.775,0.1,0.1],...
                'HorizontalAlignment','left',...
                'Foregroundcolor',[0.5,0.5,0.5],'FontWeight','bold','FontSize',[8]);

            % Input for month
            month = uicontrol(Panel_Handle,'Style','edit',...
                'String',num2str(SWFT_Configuration.month),...
                'Unit','normalized','Position',[0.175,0.85,0.05,0.1],...
                'Callback',@month_Callback);
            Panel_GHandle.month=month;            
            % Input for day
            day = uicontrol(Panel_Handle,'Style','edit',...
                'String',num2str(SWFT_Configuration.day),...
                'Unit','normalized','Position',[0.24,0.85,0.05,0.1],...
                'Callback',@day_Callback);
            Panel_GHandle.day=day;

            % Input for year
            year = uicontrol(Panel_Handle,'Style','edit',...
                'String',num2str(SWFT_Configuration.year),...
                'Unit','normalized','Position',[0.305,0.85,0.05,0.1],...
                'Callback',@year_Callback);
            Panel_GHandle.year=year;

            % Label and input for data latency.
            ldata_latency = uicontrol(Panel_Handle,'Style','text',...
                'String','Data Latency:',...
                'Unit','normalized','Position',[0.02,0.7,0.2,0.1],...
                'HorizontalAlignment','left',...
                'FontWeight','bold','FontSize',10);

            rdata_latency = uicontrol(Panel_Handle,'Style','text',...
                'String','Range = [0,10]',...
                'Unit','normalized','Position',[0.02,0.625,0.1,0.1],...
                'HorizontalAlignment','left',...
                'Foregroundcolor',[0.5,0.5,0.5],'FontWeight','bold','FontSize',[8]);

            data_latency = uicontrol(Panel_Handle,'Style','edit',...
                'String',num2str(SWFT_Configuration.latency),...
                'Unit','normalized','Position',[0.175,0.7,0.05,0.1],...
                'Callback',@data_latency_Callback);
            Panel_GHandle.data_latency=data_latency;

            % Label and input for data latency
            llead_time = uicontrol(Panel_Handle,'Style','text',...
                'String','Forcast Lead Time:',...
                'Unit','normalized','Position',[0.02,0.55,0.2,0.1],...
                'HorizontalAlignment','left',...
                'FontWeight','bold','FontSize',10);

            rlead_time = uicontrol(Panel_Handle,'Style','text',...
                'String','Range = [0,10]',...
                'Unit','normalized','Position',[0.02,0.475,0.1,0.1],...
                'HorizontalAlignment','left',...
                'Foregroundcolor',[0.5,0.5,0.5],'FontWeight','bold','FontSize',[8]);

            lead_time = uicontrol(Panel_Handle,'Style','edit',...
                'String',num2str(SWFT_Configuration.lead),...
                'Unit','normalized','Position',[0.175,0.55,0.05,0.1],...
                'Callback',@lead_time_Callback);
            Panel_GHandle.lead_time=lead_time;

            % Label and input for training interval
            ltraining_interval = uicontrol(Panel_Handle,'Style','text',...
                'String','Training Interval:',...
                'Unit','normalized','Position',[0.02,0.40,0.2,0.1],...
                'HorizontalAlignment','left',...
                'FontWeight','bold','FontSize',10);

            % rtraining_interval = uicontrol(Panel_Handle,'Style','text',...
            %     'String','Range = [1,1800]',...
            %     'Unit','normalized','Position',[0.02,0.325,0.1,0.1],...
            %     'HorizontalAlignment','left',...
            %     'Foregroundcolor',[0.5,0.5,0.5],'FontWeight','bold','FontSize',[8]);

            training_interval = uicontrol(Panel_Handle,'Style','edit',...
                'String',num2str(SWFT_Configuration.training),...
                'Unit','normalized','Position',[0.175,0.40,0.05,0.1],...
                'Callback',@training_interval_Callback);
            Panel_GHandle.training_interval=training_interval;

            % Label and input for validation interval
            lvalidation_interval = uicontrol(Panel_Handle,'Style','text',...
                'string','Validation Interval:',...
                'Unit','normalized','Position',[0.02,0.25,0.2,0.1],...
                'HorizontalAlignment','left',...
                'FontWeight','bold','FontSize',10);

            % rvalidation_interval = uicontrol(Time_Scale_Input,'Style','text',...
            %     'String','Range = [1,900]',...
            %     'Unit','normalized','Position',[0.02,0.175,0.1,0.1],...
            %     'HorizontalAlignment','left',...
            %     'Foregroundcolor',[0.5,0.5,0.5],'FontWeight','bold','FontSize',[8]);

            validation_interval = uicontrol(Panel_Handle,'Style','edit',...
                'String',num2str(SWFT_Configuration.validation),...
                'Unit','normalized','Position',[0.175,0.25,0.05,0.1],...
                'Callback',@validation_interval_Callback);
            Panel_GHandle.validation_interval=validation_interval;
        else
            Panel_GHandle=varargin{1};
            Panel_GHandle.year.String=num2str(SWFT_Configuration.year);
            Panel_GHandle.month.String=num2str(SWFT_Configuration.month);
            Panel_GHandle.day.String=num2str(SWFT_Configuration.day);
            Panel_GHandle.data_latency.String=num2str(SWFT_Configuration.latency);
            Panel_GHandle.lead_time.String=num2str(SWFT_Configuration.lead);
            Panel_GHandle.training_interval.String=num2str(SWFT_Configuration.training);
            Panel_GHandle.validation_interval.String=num2str(SWFT_Configuration.validation);
            Panel_GHandle.plotarea = plot_timescale(Panel_GHandle.Time_Scale_Plot,SWFT_Configuration.current_epoch,...
                SWFT_Configuration.latency,SWFT_Configuration.lead,...
                SWFT_Configuration.training,SWFT_Configuration.validation,Panel_GHandle.plotarea);
        end
        return
        %
        function update_forecastdata
            forecastData.DataLatency=SWFT_Configuration.latency;
            forecastData.ForecastPeriod=SWFT_Configuration.lead;
            forecastData.nTrainingDay=SWFT_Configuration.training;
            forecastData.nValidationDay=SWFT_Configuration.validation;
            forecastData.CurrentEpoc=datenum(SWFT_Configuration.year,...
                SWFT_Configuration.month,...
                SWFT_Configuration.day);
            forecastData.ForecastVar.LastTrainingDateNum=...
                forecastData.CurrentEpoc-forecastData.DataLatency;
            forecastData.ForecastVar.FirstTrainingDateNum=...
                forecastData.CurrentEpoc-forecastData.DataLatency-forecastData.nTrainingDay;
            forecastData.ForecastVar.FirstValidationDateNum=...
                forecastData.CurrentEpoc+forecastData.ForecastPeriod;
            forecastData.ForecastVar.LastValidationDateNum=...
                forecastData.CurrentEpoc+forecastData.ForecastPeriod+forecastData.nValidationDay;
            %
            forecastData.DataVar.LastTrainingDateNum=...
                forecastData.CurrentEpoc-forecastData.DataLatency-forecastData.ForecastPeriod;
            DataHistory=-inf;
            nVar=length(forecastData.DataVar.VarLabels);
            for iVar=1:nVar
                ind=strfind(forecastData.DataVar.VarLabels(iVar).VariableName,'_');
                if ~isempty(ind)
                    TimeStamp=str2num(forecastData.DataVar.VarLabels(iVar).VariableName(ind(end)+1:end));
                    if ~isempty(TimeStamp)
                        DataHistory=max([DataHistory,TimeStamp]);
                    end
                end
            end
            DataHistory=DataHistory/8;
            forecastData.DataVar.FirstTrainingDateNum=...
                forecastData.CurrentEpoc-forecastData.DataLatency-forecastData.ForecastPeriod-forecastData.nTrainingDay-DataHistory;
            forecastData.DataVar.FirstValidationDateNum=...
                forecastData.CurrentEpoc+forecastData.ForecastPeriod-forecastData.DataLatency-DataHistory;
            forecastData.DataVar.LastValidationDateNum=...
                forecastData.CurrentEpoc+forecastData.ForecastPeriod-forecastData.DataLatency+forecastData.nValidationDay;
            Variable_Selection_GHandle = creat_variable_select_panel(...
                Variable_Selection,Variable_Selection_GHandle);
            return
        end
        % Callback function for month
        function month_Callback(source,eventdata)
            monthInput = str2num(source.String);
            testingEpochNum = datenum(SWFT_Configuration.year,...
                monthInput,SWFT_Configuration.day);
            epochLowerBound = AchiveTime.Min+SWFT_Configuration.training+2*SWFT_Configuration.latency+SWFT_Configuration.lead;
            epochUpperBound = AchiveTime.Max-SWFT_Configuration.validation-SWFT_Configuration.lead;
            [f,monthValidity] = checkEpochValidity(testingEpochNum,epochLowerBound,epochUpperBound);
            if monthValidity == 1
                SWFT_Configuration.month = monthInput;
            else
                month.String = num2str(SWFT_Configuration.month);
            end
            SWFT_Configuration.current_epoch = datetime(SWFT_Configuration.year,...
                SWFT_Configuration.month,SWFT_Configuration.day);
            plotarea = plot_timescale(Time_Scale_Plot,SWFT_Configuration.current_epoch,...
                SWFT_Configuration.latency,SWFT_Configuration.lead,...
                SWFT_Configuration.training,SWFT_Configuration.validation,plotarea);
            if f ~= 0
                figure(f);
            end
            update_forecastdata;
            return
        end

        % Callback function for day
        function day_Callback(source,eventdata)
            dayInput = str2num(source.String);
            testingEpochNum = datenum(SWFT_Configuration.year,...
                SWFT_Configuration.month,dayInput);
            epochLowerBound = AchiveTime.Min+SWFT_Configuration.training+2*SWFT_Configuration.latency+SWFT_Configuration.lead;
            epochUpperBound = AchiveTime.Max-SWFT_Configuration.validation-SWFT_Configuration.lead;
            [f,dayValidity] = checkEpochValidity(testingEpochNum,epochLowerBound,epochUpperBound);
            if dayValidity == 1
                SWFT_Configuration.day = dayInput;
            else
                day.String = num2str(SWFT_Configuration.day);
            end
            SWFT_Configuration.current_epoch = datetime(SWFT_Configuration.year,...
                SWFT_Configuration.month,SWFT_Configuration.day);
            plotarea = plot_timescale(Time_Scale_Plot,SWFT_Configuration.current_epoch,...
                SWFT_Configuration.latency,SWFT_Configuration.lead,...
                SWFT_Configuration.training,SWFT_Configuration.validation,plotarea);
            if f ~= 0
                figure(f);
            end
            update_forecastdata;
            return
        end

        % Callback function for year
        function year_Callback(source,eventdata)
            yearInput = str2num(source.String);
            testingEpochNum = datenum(yearInput,...
                SWFT_Configuration.month,SWFT_Configuration.day);
            epochLowerBound = AchiveTime.Min+SWFT_Configuration.training+2*SWFT_Configuration.latency+SWFT_Configuration.lead;
            epochUpperBound = AchiveTime.Max-SWFT_Configuration.validation-SWFT_Configuration.lead;
            [f,yearValidity] = checkEpochValidity(testingEpochNum,epochLowerBound,epochUpperBound);
            if yearValidity == 1
                SWFT_Configuration.year = yearInput;
            else
                year.String = num2str(SWFT_Configuration.year);
            end
            SWFT_Configuration.current_epoch = datetime(SWFT_Configuration.year,...
                SWFT_Configuration.month,SWFT_Configuration.day);
            plotarea = plot_timescale(Time_Scale_Plot,SWFT_Configuration.current_epoch,...
                SWFT_Configuration.latency,SWFT_Configuration.lead,...
                SWFT_Configuration.training,SWFT_Configuration.validation,plotarea);
            if f ~= 0
                figure(f);
            end
            update_forecastdata;
            return
        end

        % Callback function for data latency
        function data_latency_Callback(source,eventdata)
            inputnum = str2num(source.String);
            [f,SWFT_Configuration.latency, validity] = checkvalidity(inputnum,0,10,1);
            if validity == 0
                data_latency.String = '1';
                plotarea = plot_timescale(Time_Scale_Plot,SWFT_Configuration.current_epoch,...
                SWFT_Configuration.latency,SWFT_Configuration.lead,...
                SWFT_Configuration.training,SWFT_Configuration.validation,plotarea);
                figure(f);
            else
            plotarea = plot_timescale(Time_Scale_Plot,SWFT_Configuration.current_epoch,...
                SWFT_Configuration.latency,SWFT_Configuration.lead,...
                SWFT_Configuration.training,SWFT_Configuration.validation,plotarea);
            end
            update_forecastdata;
            return
        end

        % Callback function for lead time
        function lead_time_Callback(source,eventdata)
            inputnum = str2num(source.String);
            [f,SWFT_Configuration.lead, validity] = checkvalidity(inputnum,0,10,1);
            if validity == 0
                lead_time.String = '1';
                plotarea = plot_timescale(Time_Scale_Plot,SWFT_Configuration.current_epoch,...
                SWFT_Configuration.latency,SWFT_Configuration.lead,...
                SWFT_Configuration.training,SWFT_Configuration.validation,plotarea);
                figure(f);
            else
            plotarea = plot_timescale(Time_Scale_Plot,SWFT_Configuration.current_epoch,...
                SWFT_Configuration.latency,SWFT_Configuration.lead,...
                SWFT_Configuration.training,SWFT_Configuration.validation,plotarea);
            end
            update_forecastdata;
            return
        end

        % Callback function for training interval
        function training_interval_Callback(source, eventdata)
            trainInput = str2num(source.String);
            EpochNum = datenum(SWFT_Configuration.year,...
                SWFT_Configuration.month,SWFT_Configuration.day);
            trainLowerBound = 1;
            trainUpperBound = EpochNum-AchiveTime.Min-2*SWFT_Configuration.latency-SWFT_Configuration.lead;
            trainUpperDateNum = EpochNum - 2*SWFT_Configuration.latency - SWFT_Configuration.lead;
            trainLowerDateNum = AchiveTime.Min;
            [f,trainValidity] = checkIntervalValidity(trainInput,trainLowerBound,trainUpperBound,trainLowerDateNum,trainUpperDateNum);
            if trainValidity == 1
                SWFT_Configuration.training = trainInput;
            else
                training_interval.String = num2str(SWFT_Configuration.training);
            end
            plotarea = plot_timescale(Time_Scale_Plot,SWFT_Configuration.current_epoch,...
                SWFT_Configuration.latency,SWFT_Configuration.lead,...
                SWFT_Configuration.training,SWFT_Configuration.validation,plotarea);
            if f ~= 0
                figure(f);
            end
            update_forecastdata;
            return
        end

        % Callback function for validation interval
        function validation_interval_Callback(source, eventdata)
            validationInput = str2num(source.String);
            EpochNum = datenum(SWFT_Configuration.year,...
                SWFT_Configuration.month,SWFT_Configuration.day);
            validationLowerBound = 1;
            validationUpperBound = AchiveTime.Max-EpochNum-SWFT_Configuration.lead;
            validationLowerDateNum = EpochNum + SWFT_Configuration.lead;
            validationUpperDateNum = AchiveTime.Max;
            [f,validationValidity] = checkIntervalValidity(validationInput,validationLowerBound,validationUpperBound,...
                validationLowerDateNum,validationUpperDateNum);
            if validationValidity == 1
                SWFT_Configuration.validation = validationInput;
            else
                validation_interval.String = num2str(SWFT_Configuration.validation);
            end
            plotarea = plot_timescale(Time_Scale_Plot,SWFT_Configuration.current_epoch,...
                SWFT_Configuration.latency,SWFT_Configuration.lead,...
                SWFT_Configuration.training,SWFT_Configuration.validation,plotarea);
            if f ~= 0
                figure(f);
            end
            update_forecastdata;
            return
        end
    end
%
% end of creating time scale panel and its sub-functions.
%
% Create variable selction panel and its sub-functions.
%
    function [Panel_GHandle] = creat_variable_select_panel(Panel_Handle,varargin)
        Column_Width=0.185;
        Column_Space=0.015;
        if nargin == 1
            %
            % Creating the panel for the first time.
            %
            Panel_GHandle = struct('Panel',Panel_Handle);
            % Push buttons
            forecast_pushbutton = uicontrol(Panel_Handle,'Style','pushbutton',...
                'String','Forecast','Unit','normalized','Position',[0.02,0.925,0.08,0.05],...
                'BackgroundColor',[0.3,0.8,0.1],'Callback',@forecast_pushbutton_Callback);
            Panel_GHandle.forecast_pushbutton = forecast_pushbutton;
            
            covariate_pushbutton = uicontrol(Panel_Handle,'Style','pushbutton',...
                'String','Covariates','Unit','normalized','Position',[0.12,0.925,0.08,0.05],...
                'Callback',@covariate_pushbutton_Callback);
            Panel_GHandle.covariate_pushbutton = covariate_pushbutton;
            
            explore_pushbutton = uicontrol(Panel_Handle,'Style','pushbutton',...
                'String','Explore','Unit','normalized','Position',[0.22,0.925,0.08,0.05],...
                'Callback',@explore_pushbutton_Callback);
            Panel_GHandle.explore_pushbutton = explore_pushbutton;
            %
            %
            listbox1_title = uicontrol(Panel_Handle,'Style','text',...
                'String','Data Collection',...
                'Unit','normalized','Position',[0.02,0.8,0.15,0.1],...
                'FontWeight','bold','HorizontalAlignment','left','FontSize',12);
            
            listbox2_title = uicontrol(Panel_Handle,'Style','text',...
                'String','Variables','Unit','normalized',...
                'Position',[0.02+Column_Width+Column_Space,0.8,0.15,0.1],...
                'FontWeight','bold','HorizontalAlignment','left','FontSize',12);
            
            listbox3_title = uicontrol(Panel_Handle,'Style','text',...
                'String','Select Local Attributes','Unit','normalized',...
                'Position',[0.02+2*Column_Width+2*Column_Space,0.8,0.2,0.1],...
                'FontWeight','bold','HorizontalAlignment','left','FontSize',12);
            
            listbox4_title = uicontrol(Panel_Handle,'Style','text',...
                'String','Final Selection','Unit','normalized',...
                'Position',[0.02+3*Column_Width+3*Column_Space,0.8,0.2,0.1],...
                'FontWeight','bold','HorizontalAlignment','left','FontSize',12);
            
            listbox5_title = uicontrol(Panel_Handle,'Style','text',...
                'String','Covariate Time Sample','Unit','normalized','Visible','off',...
                'Position',[0.02+4*Column_Width+4*Column_Space,0.8,0.2,0.1],...
                'FontWeight','bold','HorizontalAlignment','left','FontSize',12);
            %
            [DataCategory] = getSWFT_DataCategory(SWFTData_Path);
            [VarInfo] = getVarInfo(forecastData.ForecastVar.VarLabels,SWFTData_Path);
            FirstVar=VarInfo(1);
            Cat_Value = find(strcmp(DataCategory,FirstVar.Category)==1);
            listbox1 = uicontrol(Panel_Handle,'Style','listbox',...
                'Unit','normalized','Position',[0.02,0.025,Column_Width,0.8],...
                'String',DataCategory,'Value',Cat_Value,...
                'Callback',@listbox1_Callback,'ButtonDownFcn',@listbox1_ButtonDownFcn);
            Panel_GHandle.listbox1 = listbox1;
            
            [VariableList] = getSWFT_Variables(SWFTData_Path,DataCategory{Cat_Value});
            Var_Value = find(strcmp(unique({VariableList(:).NativeName}),FirstVar.NativeName));
            NativeName=unique({VariableList(:).NativeName});
            listbox2 = uicontrol(Panel_Handle,'Style','listbox',...
                'Unit','normalized','Position',[0.02+Column_Width+Column_Space,0.025,...
                Column_Width,0.8],'Callback', @listbox2_Callback,...
                'Enable','on','ButtonDownFcn',@listbox2_ButtonDownFcn);
            listbox2.String = NativeName;
            listbox2.Value = Var_Value;
            Panel_GHandle.listbox2 = listbox2;

            varIndex = find(strcmp({VariableList(:).NativeName},FirstVar.NativeName)==1);
            Loc_Value = find(strcmp({VariableList(varIndex).VarName},FirstVar.VarName)==1);
            listbox3 = uicontrol(Panel_Handle,'Style','listbox',...
                'Unit','normalized','Position',[0.02+2*(Column_Width+Column_Space),0.025,...
                Column_Width,0.8],'Callback', @listbox3_Callback,...
                'Enable','on','ButtonDownFcn',@listbox3_ButtonDownFcn);
            listbox3.String = {VariableList(varIndex).VarName};
            listbox3.Value = Loc_Value;
            Panel_GHandle.listbox3 = listbox3;
            
            listbox4 = uicontrol(Panel_Handle,'Style','listbox',...
                'Unit','normalized','Position',[0.02+3*(Column_Width+Column_Space),0.025,...
                Column_Width,0.8],'Callback', @listbox4_Callback,...
                'Enable','on','ButtonDownFcn',@listbox4_ButtonDownFcn);
            listbox4.String = {[listbox1.String{Cat_Value},'/',...
                    forecastData.ForecastVar.VarLabels(:).VariableName]};
            listbox4.Value = 1;
            Panel_GHandle.listbox4 = listbox4;
            
            delete_pushbutton = uicontrol(Panel_Handle,'Style','pushbutton',...
                'String','Delete','Unit','normalized','Position',...
                [0.02+4*Column_Width+3*Column_Space-0.05,0.84,0.05,0.05],...
                'Callback',@delete_Callback);
            Panel_GHandle.delete_pushbutton = delete_pushbutton;
            
            % The last column
            all_variable = uicontrol(Panel_Handle,'Style','radiobutton',...
                'Unit','normalized','String','For all variables','FontSize',11,...
                'Position',[0.02+4*(Column_Width+Column_Space),0.75,0.2,0.08],...
                'Visible','off','Callback', @all_variable_Callback);
            Panel_GHandle.all_variable=all_variable;
            
            selected_variable = uicontrol(Panel_Handle,'Style','radiobutton',...
                'Unit','normalized','String','For selected variables','FontSize',11,...
                'Position',[0.02+4*(Column_Width+Column_Space),0.67,0.2,0.08],...
                'Visible','off','Callback', @selected_variable_Callback);
            Panel_GHandle.selected_variable=selected_variable;
            
            lmost_recent_data = uicontrol(Panel_Handle,'Style','text','Visible','off',...
                'Unit','normalized','String','Most Recent Covariate Data','FontSize',11,...
                'HorizontalAlignment','left','FontWeight','bold',...
                'Position',[0.02+4*(Column_Width+Column_Space),0.58,0.17,0.08]);
            
            most_recent_data = uicontrol(Panel_Handle,'Style','text','Unit','normalized',...
                'HorizontalAlignment','left','Visible','off','FontSize',11,'Position',...
                [0.02+4*(Column_Width+Column_Space),0.5,0.17,0.08]);
            Panel_GHandle.most_recent_data=most_recent_data;

            ladd_data = uicontrol(Panel_Handle,'Style','text','Visible','off',...
                'Unit','normalized','string','Add Older Data','FontSize',11,...
                'HorizontalAlignment','left','FontWeight','bold',...
                'Position',[0.02+4*(Column_Width+Column_Space),0.42,0.1,0.08]);
            
            add_data = uicontrol(Panel_Handle,'Style','edit','Unit','normalized',...
                'HorizontalAlignment','left','Visible','off','Position',...
                [0.02+4*(Column_Width+Column_Space),0.34,0.17,0.08],...
                'Callback',@add_data_Callback);
            Panel_GHandle.add_data=add_data;
            
            Help_pushbutton = uicontrol(Panel_Handle,'Style','pushbutton',...
                'String','Help','Unit','normalized','Visible','off',...
                'Position',[0.02+4*(Column_Width+Column_Space),0.24,0.08,0.05],...
                'Callback',@help_pushbutton_Callback);
            Panel_GHandle.Variable_Type = 'forecast';
        else
            %
            % When the selction is switched between forecast variables and
            % covariate variables. The panels are initialized with the
            % category of the first variable highlighted.
            %
            Panel_GHandle = varargin{1};
            [DataCategory] = getSWFT_DataCategory(SWFTData_Path);
            [VarInfo] = getVarInfo(forecastData.ForecastVar.VarLabels,SWFTData_Path);
            Panel_GHandle.Variable_Type = 'forecast';
            %
            % The color of the button is used to determine whether the user
            % is selection forecast variable or covariates.
            %
            if Panel_GHandle.forecast_pushbutton.BackgroundColor(1) == 0.3
                [VarInfo] = getVarInfo(forecastData.ForecastVar.VarLabels,SWFTData_Path);
                Panel_GHandle.Variable_Type = 'forecast';
            elseif Panel_GHandle.covariate_pushbutton.BackgroundColor(1) == 0.3
                [VarInfo] = getVarInfo(forecastData.DataVar.VarLabels,SWFTData_Path);
                Panel_GHandle.Variable_Type = 'covariate';
            end
            FirstVar=VarInfo(1);
            Cat_Value = find(strcmp(DataCategory,FirstVar.Category)==1);
            Panel_GHandle.listbox1.String = DataCategory;
            Panel_GHandle.listbox1.Value = Cat_Value;
            %
            [VariableList] = getSWFT_Variables(SWFTData_Path,DataCategory{Cat_Value});
            Var_Value=find(strcmp(unique({VariableList(:).NativeName}),FirstVar.NativeName));
            NativeName=unique({VariableList(:).NativeName});
            Panel_GHandle.listbox2.String = NativeName;
            Panel_GHandle.listbox2.Value = Var_Value;
            %
            varIndex = find(strcmp({VariableList(:).NativeName},FirstVar.NativeName)==1);
            Loc_Value=find(strcmp({VariableList(varIndex).VarName},FirstVar.VarName)==1);
            Panel_GHandle.listbox3.String = {VariableList(varIndex).VarName};
            Panel_GHandle.listbox3.Value = Loc_Value;
            %
            VarList =cell(length(VarInfo),1);
            for i = 1:length(VarInfo)
                VarList{i} = [VarInfo(i).Category,'/',VarInfo(i).VarName];
            end
            VarList = unique(VarList);
            Panel_GHandle.listbox4.String=VarList;
            varIndex=find(strcmp(VarList,[FirstVar.Category,'/',FirstVar.VarName])==1);
            Panel_GHandle.listbox4.Value=varIndex;
            %
            if strcmp(Panel_GHandle.Variable_Type,'covariate')
                %
                %  The add time data sample panel is initialized to the
                %  first variable in the category.
                %
                varIndex=find(strcmp({VarInfo(:).VarName},FirstVar.VarName)==1);
                TimeStamp=num2str(VarInfo(varIndex(1)).TimeStamp);
                for i=2:length(varIndex)
                    TimeStamp=[TimeStamp,', ',num2str(VarInfo(varIndex(i)).TimeStamp)];                    
                end
                Panel_GHandle.add_data.String = TimeStamp;
                Panel_GHandle.selected_variable.Value=1;
                Panel_GHandle.most_recent_data.String=[datestr(forecastData.CurrentEpoc-...
                    forecastData.DataLatency - forecastData.ForecastPeriod,...
                    'mm/dd/yyyy'),' 00:00:00'];
            end
        end
        return
        %
        % Callback functions
        %
        function forecast_pushbutton_Callback(source,eventdata)
            if source.BackgroundColor(1) == 0.94
                source.BackgroundColor = [0.3,0.8,0.1];
                covariate_pushbutton.BackgroundColor = [0.94 0.94 0.94];
                explore_pushbutton.BackgroundColor = [0.94 0.94 0.94];
            else
                source.BackgroundColor = [0.94 0.94 0.94];
            end
            listbox5_title.Visible = 'off';
            all_variable.Visible = 'off';
            selected_variable.Visible = 'off';
            lmost_recent_data.Visible = 'off';
            most_recent_data.Visible = 'off';
            ladd_data.Visible = 'off';
            add_data.Visible = 'off';
            Help_pushbutton.Visible = 'off';
            Variable_Selection_GHandle = creat_variable_select_panel(...
                Variable_Selection,Variable_Selection_GHandle);
        end
        
        function covariate_pushbutton_Callback(source,eventdata)
            if source.BackgroundColor(1) == 0.94
                source.BackgroundColor = [0.3,0.8,0.1];
                forecast_pushbutton.BackgroundColor = [0.94 0.94 0.94];
                explore_pushbutton.BackgroundColor = [0.94 0.94 0.94];
            else
                source.BackgroundColor = [0.94 0.94 0.94];
            end
            listbox5_title.Visible = 'on';
            all_variable.Visible = 'on';
            selected_variable.Visible = 'on';
            lmost_recent_data.Visible = 'on';
            most_recent_data.Visible = 'on';
            ladd_data.Visible = 'on';
            add_data.Visible = 'on';
            Help_pushbutton.Visible = 'on';
            Variable_Selection_GHandle = creat_variable_select_panel(...
                Variable_Selection,Variable_Selection_GHandle);
        end
        %
        function explore_pushbutton_Callback(source,eventdata)
            if source.BackgroundColor(1) == 0.94
                source.BackgroundColor = [0.3,0.8,0.1];
                forecast_pushbutton.BackgroundColor = [0.94 0.94 0.94];
                covariate_pushbutton.BackgroundColor = [0.94 0.94 0.94];
            else
                source.BackgroundColor = [0.94 0.94 0.94];
            end
        end
        %
        % Callback functions for listboxs
        %
        function listbox1_Callback(source,eventdata)
            %
            % A data category is selected. No new variable is added.
            %
            [DataCategory_loc] = getSWFT_DataCategory(SWFTData_Path);
            [VariableList_loc] = getSWFT_Variables(SWFTData_Path,DataCategory_loc{source.Value});
            listbox2.String = unique({VariableList_loc(:).NativeName});
            if strcmp(Variable_Selection_GHandle.Variable_Type,'forecast')
                [VarInfo_loc] = getVarInfo(forecastData.ForecastVar.VarLabels,SWFTData_Path);
            else
                [VarInfo_loc] = getVarInfo(forecastData.DataVar.VarLabels,SWFTData_Path);
            end
            if isempty(VarInfo_loc)
                VarInfo_loc=struct('VarName',cell(1,0),'NativeName',cell(1,0),'Category',cell(1,0));
            end
            % 
            % Find variables in the specified category. If it is a new
            % category, initialize the column by default.
            %
            Var_Index_loc = find(strcmp({VarInfo_loc(:).Category},DataCategory_loc{source.Value})==1);
            if isempty(Var_Index_loc)
                listbox2.Value = 1;
                Var_Index_1 = find(strcmp({VariableList_loc(:).NativeName},listbox2.String{1})==1);
                listbox3.String = unique({VariableList_loc(Var_Index_1).VarName});
                listbox3.Value = 1;
                if strcmp(Variable_Selection_GHandle.Variable_Type,'covariate')
                    add_data.String = [];
                    selected_variable.Value=0;
                    most_recent_data.String=[datestr(forecastData.CurrentEpoc-...
                        forecastData.DataLatency - forecastData.ForecastPeriod,...
                        'mm/dd/yyyy'),' 00:00:00'];
                end
            else
                %
                % If there is a variable in the selected category, panel
                % 2-4 are initialized for that variable.
                %
                FirstVar_0=VarInfo_loc(Var_Index_loc(1));
                listbox2.Value=find(strcmp(listbox2.String,FirstVar_0.NativeName)==1);
                Var_Index_1 = find(strcmp({VariableList_loc(:).NativeName},FirstVar_0.NativeName)==1);
                Loc_Value_loc=find(strcmp({VariableList_loc(Var_Index_1).VarName},FirstVar_0.VarName)==1);
                listbox3.String = unique({VariableList_loc(Var_Index_1).VarName});
                listbox3.Value = Loc_Value_loc;
                VarList_loc =cell(length(VarInfo_loc),1);
                for j = 1:length(VarInfo_loc)
                    VarList_loc{j} = [VarInfo_loc(j).Category,'/',VarInfo_loc(j).VarName];
                end
                VarList_loc = unique(VarList_loc);
                listbox4.String=VarList_loc;
                Var_Index_2=find(strcmp(VarList_loc,[FirstVar_0.Category,'/',FirstVar_0.VarName])==1);
                listbox4.Value=Var_Index_2;
                if strcmp(Variable_Selection_GHandle.Variable_Type,'covariate')
                    Var_Index_3=find(strcmp({VarInfo_loc(:).VarName},FirstVar_0.VarName)==1);
                    TimeStamp_loc=num2str(VarInfo_loc(Var_Index_3(1)).TimeStamp);
                    for j=2:length(Var_Index_3)
                        TimeStamp_loc=[TimeStamp_loc,', ',num2str(VarInfo_loc(Var_Index_3(j)).TimeStamp)];
                    end
                    add_data.String = TimeStamp_loc;
                    all_variable.Value=0;
                    selected_variable.Value=1;
                    most_recent_data.String=[datestr(forecastData.CurrentEpoc-...
                        forecastData.DataLatency - forecastData.ForecastPeriod,...
                        'mm/dd/yyyy'),' 00:00:00'];
                end
            end
        end
       
        %
        %
        function listbox2_Callback(source,eventdata)
            %
            % A native variable is selected. No new variable is added.
            %
            if strcmp(Variable_Selection_GHandle.Variable_Type,'forecast')
                [VarInfo_loc] = getVarInfo(forecastData.ForecastVar.VarLabels,SWFTData_Path);
            else
                [VarInfo_loc] = getVarInfo(forecastData.DataVar.VarLabels,SWFTData_Path);
            end
            if isempty(VarInfo_loc)
                VarInfo_loc=struct('VarName',cell(1,0),'NativeName',cell(1,0),'Category',cell(1,0));
            end
            [DataCategory_loc] = getSWFT_DataCategory(SWFTData_Path);
            [VariableList_loc] = getSWFT_Variables(SWFTData_Path,DataCategory_loc{listbox1.Value});
            %
            % Initialize 3dr column all variables with the same native
            % name.
            %
            Var_Ind_0 = find(strcmp({VariableList_loc(:).NativeName},source.String{source.Value})==1);
            listbox3.String = unique({VariableList_loc(Var_Ind_0).VarName});
            %
            % Find existing variables with the same category and native
            % name.
            %
            Var_Index_loc = find((strcmp({VarInfo_loc(:).Category},DataCategory_loc{listbox1.Value})==1)&...
                (strcmp({VarInfo_loc(:).NativeName},source.String{source.Value})==1));
            if isempty(Var_Index_loc)
                %
                % If no existing variable, initialize by default.
                %
                listbox3.Value = 1;
                if strcmp(Variable_Selection_GHandle.Variable_Type,'covariate')
                    add_data.String = [];
                    selected_variable.Value=1;
                    all_variable.Value=0;
                    most_recent_data.String=[datestr(forecastData.CurrentEpoc-...
                        forecastData.DataLatency - forecastData.ForecastPeriod,...
                        'mm/dd/yyyy'),' 00:00:00'];
                end
            else
                %
                % Initialize column 3, 4 and time stamps by the first
                % existing variable in the category, with the native name
                % and local statistic.
                %
                FirstVar_0=VarInfo_loc(Var_Index_loc(1));
                Var_Ind_0=find(strcmp(listbox3.String,FirstVar_0.VarName)==1);
                listbox3.Value = Var_Ind_0;
                Var_Ind_1=find(strcmp(listbox4.String,[FirstVar_0.Category,'/',...
                    FirstVar_0.VarName])==1);
                listbox4.Value = Var_Ind_1;
                if strcmp(Variable_Selection_GHandle.Variable_Type,'covariate')
                    Var_Index_3=find(strcmp({VarInfo_loc(:).VarName},FirstVar_0.VarName)==1);
                    TimeStamp_loc=num2str(FirstVar_0.TimeStamp);
                    for j=2:length(Var_Index_3)
                        TimeStamp_loc=[TimeStamp_loc,', ',num2str(VarInfo_loc(Var_Index_3(j)).TimeStamp)];
                    end
                    add_data.String = TimeStamp_loc;
                    selected_variable.Value=1;
                    most_recent_data.String=[datestr(forecastData.CurrentEpoc-...
                        forecastData.DataLatency - forecastData.ForecastPeriod,...
                        'mm/dd/yyyy'),' 00:00:00'];
                end
            end
        end
        %
        % 
        function listbox3_Callback(source,eventdata)
            %
            % A local statiscs is selected. A new variable is potentially
            % added.
            %
            if strcmp(Variable_Selection_GHandle.Variable_Type,'forecast')
                [VarInfo_loc] = getVarInfo(forecastData.ForecastVar.VarLabels,SWFTData_Path);
            else
                [VarInfo_loc] = getVarInfo(forecastData.DataVar.VarLabels,SWFTData_Path);
            end
            newVar = [listbox1.String{listbox1.Value},'/',source.String{source.Value}];
            if ~any(strcmp(listbox4.String,newVar))
                listbox4.String{end+1} = newVar;
                listbox4.Value=length(listbox4.String);
                if strcmp(Variable_Selection_GHandle.Variable_Type,'covariate')
                    add_data.String = '1';
                    selected_variable.Value=1;
                    all_variable.Value=0;
                    most_recent_data.String=[datestr(forecastData.CurrentEpoc-...
                        forecastData.DataLatency - forecastData.ForecastPeriod,...
                        'mm/dd/yyyy'),' 00:00:00'];
                end
                if strcmp(Variable_Selection_GHandle.Variable_Type,'covariate')
                    [forecastData.DataVar.VarLabels]=add_variable(forecastData.DataVar.VarLabels,...
                        listbox1.String{listbox1.Value},[listbox3.String{listbox3.Value},'_1'],...
                        SWFTData_Path);
                else
                    [forecastData.ForecastVar.VarLabels]=add_variable(forecastData.ForecastVar.VarLabels,...
                        listbox1.String{listbox1.Value},listbox3.String{listbox3.Value},...
                        SWFTData_Path);
                end
            else
                Var_Index_loc = find((strcmp({VarInfo_loc(:).VarName},source.String{source.Value})==1));
                FirstVar_0=VarInfo_loc(Var_Index_loc(1));
                listbox4.Value=find(strcmp(listbox4.String,[FirstVar_0.Category,'/',FirstVar_0.VarName])==1);
                if strcmp(Variable_Selection_GHandle.Variable_Type,'covariate')
                    TimeStamp_loc=num2str(VarInfo_loc(Var_Index_loc(1)).TimeStamp);
                    for j=2:length(Var_Index_loc)
                        TimeStamp_loc=[TimeStamp_loc,', ',num2str(VarInfo_loc(Var_Index_loc(j)).TimeStamp)];
                    end
                    add_data.String = TimeStamp_loc;
                    selected_variable.Value=1;
                    all_variable.Value=0;
                    most_recent_data.String=[datestr(forecastData.CurrentEpoc-...
                        forecastData.DataLatency - forecastData.ForecastPeriod,...
                        'mm/dd/yyyy'),' 00:00:00'];
                end
            end
        end
        %
        %
        function listbox4_Callback(source,eventdata)
            if strcmp(Variable_Selection_GHandle.Variable_Type,'forecast')
                [VarInfo_loc] = getVarInfo(forecastData.ForecastVar.VarLabels,SWFTData_Path);
            else
                [VarInfo_loc] = getVarInfo(forecastData.DataVar.VarLabels,SWFTData_Path);
            end
            VarItem=source.String{source.Value};
            ind=strfind(VarItem,'/');
            VarCategory=VarItem(1:ind-1);
            VarName=VarItem(ind+1:end);
            Var_Ind_0=find((strcmp({VarInfo_loc(:).Category},VarCategory)==1) &...
                (strcmp({VarInfo_loc(:).VarName},VarName)==1));
            if delete_pushbutton.BackgroundColor(1) ~= 0.94
                %
                % If delete button is selected, the current variable is
                % removed from the list.
                %
                Var_Ind_1=[1:length(VarInfo_loc)];
                Var_Ind_1(Var_Ind_0)=NaN;    
                if strcmp(Variable_Selection_GHandle.Variable_Type,'forecast')
                    forecastData.ForecastVar.VarLabels=...
                        forecastData.ForecastVar.VarLabels(~isnan(Var_Ind_1));
                else
                    forecastData.DataVar.VarLabels=...
                        forecastData.DataVar.VarLabels(~isnan(Var_Ind_1));
                end                
                source.String(strcmp(source.String,source.String{source.Value})) = [];
                if source.Value > 1
                    source.Value = source.Value - 1;
                end
            elseif strcmp(Variable_Selection_GHandle.Variable_Type,'covariate')
                %
                % Change the add variable panel
                %
                %  The add time data sample panel is initialized to the
                %  first variable in the category.
                %
                Var_Index=find(strcmp({VarInfo_loc(:).VarName},VarInfo_loc(Var_Ind_0(1)).VarName)==1);
                TimeStamp_loc=num2str(VarInfo_loc(Var_Index(1)).TimeStamp);
                for i_loc=2:length(Var_Index)
                    TimeStamp_loc=[TimeStamp_loc,', ',num2str(VarInfo_loc(Var_Index(i_loc)).TimeStamp)];                    
                end
                add_data.String = TimeStamp_loc;
                selected_variable.Value=1;
                all_variable.Value=0;
                most_recent_data.String=[datestr(forecastData.CurrentEpoc-...
                    forecastData.DataLatency - forecastData.ForecastPeriod,...
                    'mm/dd/yyyy'),' 00:00:00'];
            end
            if isempty(source.String)
                delete_pushbutton.BackgroundColor = [0.94 0.94 0.94];
            end
        end
        
        function delete_Callback(source,eventdata)
            if source.BackgroundColor(1) == 0.94
                source.BackgroundColor = [0.9,0.2,0];
            else
                source.BackgroundColor = [0.94 0.94 0.94];
            end
        end
        % Callback functions
        function all_variable_Callback(source,eventdata)
            if source.Value == source.Max
                selected_variable.Value = source.Min;
                add_data.String = [];
            end
            most_recent_data.String = [datestr(forecastData.CurrentEpoc...
                - forecastData.DataLatency - forecastData.nTrainingDay,...
                'mm/dd/yyyy'),' 00:00:00'];
        end
        %
        function selected_variable_Callback(source,eventdata)
            if source.Value == source.Max
                all_variable.Value = source.Min;
                add_data.String = [];
            end            
            most_recent_data.String = [datestr(forecastData.CurrentEpoc...
                - forecastData.DataLatency - forecastData.nTrainingDay,...
                'mm/dd/yyyy'),' 00:00:00'];
        end
        % 
        function add_data_Callback(source,eventdata)
            %
            % Time stamps can only be added for cpvariate variables.
            %
            try
                TimeStamps=eval(['[',source.String,']']);
            catch
                source.String='Invalid Time Stamp';
                return;                
            end
            if isempty(TimeStamps)
                source.String='Invalid Time Stamp';
                return;
            end
            if ~isnumeric(TimeStamps)
                source.String='Invalid Time Stamp';
                return;
            end
            TimeStamps=unique(TimeStamps);
            if TimeStamps(1)<0
                source.String='Invalid Time Stamp';
                return;
            end
            [VarInfo_loc] = getVarInfo(forecastData.DataVar.VarLabels,SWFTData_Path);
            %
            if all_variable.Value == all_variable.Max
                % Button for all variable is on
                nVar=length(listbox4.String);
                forecastData.DataVar.VarLabels=forecastData.DataVar.VarLabels([]);
                for iVar=1:nVar
                    VarItem=listbox4.String{iVar};
                    ind=strfind(VarItem,'/');
                    VarCategory=VarItem(1:ind-1);
                    VarName=VarItem(ind+1:end);
                    for iTime=1:length(TimeStamps)
                        [forecastData.DataVar.VarLabels]=add_variable(forecastData.DataVar.VarLabels,...
                            VarCategory,[VarName,'_',num2str(TimeStamps(iTime))],SWFTData_Path);
                    end
                end
            elseif selected_variable.Value == selected_variable.Max
                % Button for selected variable is on
                VarItem=listbox4.String{listbox4.Value};
                ind=strfind(VarItem,'/');
                VarCategory=VarItem(1:ind-1);
                VarName=VarItem(ind+1:end);
                Var_Ind_0=find((strcmp({VarInfo_loc(:).Category},VarCategory)==1) &...
                    (strcmp({VarInfo_loc(:).VarName},VarName)==1));
                Var_Ind_1=[1:length(VarInfo_loc)];
                Var_Ind_1(Var_Ind_0)=NaN;
                NewVar=VarInfo_loc(Var_Ind_0(1));
                forecastData.DataVar.VarLabels=forecastData.DataVar.VarLabels(~isnan(Var_Ind_1));
                for iTime=1:length(TimeStamps)
                    [forecastData.DataVar.VarLabels]=add_variable(forecastData.DataVar.VarLabels,...
                        NewVar.Category,[NewVar.VarName,'_',num2str(TimeStamps(iTime))],SWFTData_Path);
                end
            end
        end
        %
        % Button down call back
        function create_SWFT_diagbox(Document_Text)
            gHandle=helpdlg(Document_Text,'SWFT Data Info');
            set(gHandle,'Position',[600,600,400,600]);
            eHandle=get(gHandle,'Children');
            set(eHandle(2),'Unit','normalized','Position',[0.05,0.85,0.1,0.075]);
            set(eHandle(3),'Unit','normalized','Position',[0.1,0.1,0.7,0.7]);
            tHandle=get(eHandle(3),'Children');
            delete(tHandle);
            subplot(eHandle(3));
            tHandle=text('Unit','normalized','Position',[0,1],'String',Document_Text,...
            'FontSize',[20],'VerticalALignment','top','Interpreter','none');
            return;
        end
        function listbox1_ButtonDownFcn(source,eventdata)
            create_SWFT_diagbox({'Current Categories:',...
                ' ',...
                'GIM Features:',...
                '   Feature extracted from GIM maps',...
                ' ',...
                'Space Physics:',...
                '   Space physics measurements or',...
                '   indices',...
                ' ',...
                'GIM Anomaly Flags:',...
                '   Anomaly flags based on GIM',...
                '   features',...
                ' ',...
                'Space Physics Anomaly:',...
                '   ANomaly flags based on space'...
                '   physics measurements or indices'});
            return;
        end
        function listbox2_ButtonDownFcn(source,eventdata)
            %
            Value=get(source,'Value');
            ItemList=get(source,'String');
            %
            create_SWFT_diagbox({'Variable:',...
                ' ',...
                ItemList{Value}});
            return;
        end
        function listbox3_ButtonDownFcn(source,eventdata)
            Value=get(source,'Value');
            ItemList=get(source,'String');
            %
            create_SWFT_diagbox({'Local Statistics',...
                'for Variable:',...
                ' ',...
                ItemList{Value}});
            return;
        end
        function listbox4_ButtonDownFcn(source,eventdata)
            Value=get(source,'Value');
            ItemList=get(source,'String');
            %
            create_SWFT_diagbox({'Selected Variable',...
                'and Local Stats:',...
                ' ',...
                ItemList{Value}});
            return;
        end
    end
    %
    %
    function [Panel_GHandle]=create_action_panel(Panel_Handle,varargin)
        if nargin == 1
            Panel_GHandle = struct('Panel',Panel_Handle);
            MLReg_pushbutton = uicontrol(Panel_Handle,'Style','pushbutton',...
                'String','Multilinear Regression','Unit','normalized','Position',[0.02,0.5,0.08,0.2],...
                'Callback',@MLReg_pushbutton_Callback);
            Panel_GHandle.MLReg_pushbutton = MLReg_pushbutton;
            
            LGReg_pushbutton = uicontrol(Panel_Handle,'Style','pushbutton',...
                'String','Logistic Regression','Unit','normalized','Position',[0.12,0.5,0.08,0.2],...
                'Callback',@LGReg_pushbutton_Callback);
            Panel_GHandle.LGReg_pushbutton = LGReg_pushbutton;
            
            RTree_pushbutton = uicontrol(Panel_Handle,'Style','pushbutton',...
                'String','Random Tree','Unit','normalized','Position',[0.22,0.5,0.08,0.2],...
                'Callback',@RTree_pushbutton_Callback);
        else
            Panel_GHandle = varargin{1};
        end
        return;
        function MLReg_pushbutton_Callback(source,eventdata)
            [forecastData]=UpdateForecastData(forecastData,SWFTData_Path);
            [forecastData]=multilinearReg(forecastData);
            visualizeMultiLinearReg(forecastData);
            return;
        end
        function LGReg_pushbutton_Callback(source,eventdata)
            [forecastData]=UpdateForecastData(forecastData,SWFTData_Path);
            [forecastData]=LogisticReg(forecastData);
            visualizeLogisticReg(forecastData);
            return;
        end
        function RTree_pushbutton_Callback(source,eventdata)
            [forecastData]=UpdateForecastData(forecastData,SWFTData_Path);
            [forecastData]=RandomTree(forecastData);
            %visualizeRandomTree(forecastData);
            return;
        end
    end
end