function [gHandle]=plot_timescale(ax_hanlde,epoch,latency,lead,training,validation,varargin)
%
% Create a plot in the axes.
%
axes(ax_hanlde);
set(gca,'YTickLabel',[]);
%        
gHandle=[];
if nargin > 6
    gHandle=varargin{1};
end
bar_width = 1;

epoch_initial = epoch;
latency_initial = latency;
lead_initial = lead;
training_initial = training;
validation_initial = validation;

% Epoch = 0 for the plot.
epochPlot = 0;

% Change into arctan
% epoch = round(atan(0.1*epochPlot) * 50);
% latency = round(atan(0.1*latency) * 50);
% lead = round(atan(0.1*lead) * 50);
% training = round(atan(0.1*training) * 50);
% validation = round(atan(0.1*validation) * 50);
    function y = powerfunc(x)
        if x < 20
            y = round(1.6.*x);
        else
            a = 13.24;
            b = 0.29;

            y = round(a.*x.^b);
        end
        return
    end

epoch = powerfunc(epochPlot);
latency = powerfunc(latency);
lead = powerfunc(lead);
training = powerfunc(training);
validation = powerfunc(validation);

% Set the color for each bar
latency_color = [0.9,0.8,0.6];
lead_color = [0.9,0.9,0.8];
training_color = [0.55,0.7,0.85];
trainingUsed_color = [0.3,0.5,0.8];
validation_color = [0.65,0.85,0.55];
validationUsed_color = [0.2,0.7,0.2];

% What's this?
if isempty(gHandle)
    gHandle = struct('fig',gcf,'axes',gca);
end

% Latency
xLatency = epoch-[0,latency,latency,0];
yLatency = [0,0,2*bar_width,2*bar_width];
if ~isfield(gHandle,'latency')
    gHandle.latency=patch(xLatency, yLatency, latency_color);
    gHandle.latencyText = text(mean(xLatency),mean(yLatency),'Latency','HorizontalAlignment','center',...
        'VerticalAlignment','middle','Rotation',90);
else
    gHandle.latency.Vertices=[xLatency',yLatency'];
    gHandle.latencyText.Position = [mean(xLatency),mean(yLatency)];
end

% Lead
xLead = epoch+[0,lead,lead,0];
yLead = [0,0,2*bar_width,2*bar_width];
if ~isfield(gHandle,'lead')
    gHandle.lead=patch(xLead, yLead, lead_color);
    gHandle.leadText = text(mean(xLead),mean(yLead),'Lead','HorizontalAlignment','center',...
        'VerticalAlignment','middle','Rotation',90);
else
    gHandle.lead.Vertices=[xLead',yLead'];
    gHandle.leadText.Position = [mean(xLead),mean(yLead)];
end

% Training
xTraining = epoch-latency-[0,training,training,0];
yTraining = [bar_width,bar_width,2*bar_width,2*bar_width];
if ~isfield(gHandle,'training')
    gHandle.training=patch(xTraining, yTraining, training_color);
    gHandle.trainingText = text(mean(xTraining),mean(yTraining),'Training','HorizontalAlignment','center',...
        'VerticalAlignment','middle');
else
    gHandle.training.Vertices=[xTraining',yTraining'];
    gHandle.trainingText.Position = [mean(xTraining),mean(yTraining)];
end
%xtrainingText = xtraining(2);
%ytrainingText = 0.5 * (ytraining(3) + ytraining(2));
%text(xtrainingText,ytrainingText,'training Interval','Color','red',...
%'FontWeight','bold')

% TrainingUsed
xTrainingUsed = epoch-2*latency-lead-[0,training,training,0];
yTrainingUsed = [0,0,bar_width,bar_width];
if ~isfield(gHandle,'trainingUsed')
    gHandle.trainingUsed=patch(xTrainingUsed, yTrainingUsed, trainingUsed_color);
    gHandle.trainingUsedText = text(mean(xTrainingUsed),mean(yTrainingUsed),'Training Used','HorizontalAlignment','center',...
        'VerticalAlignment','middle');
else
    gHandle.trainingUsed.Vertices=[xTrainingUsed',yTrainingUsed'];
    gHandle.trainingUsedText.Position = [mean(xTrainingUsed),mean(yTrainingUsed)];
end

% Validation
xValidation = epoch+lead+[0,validation,validation,0];
yValidation = [3*bar_width,3*bar_width,4*bar_width,4*bar_width];
if ~isfield(gHandle,'validation')
    gHandle.validation=patch(xValidation, yValidation, validation_color);
    gHandle.validationText = text(mean(xValidation),mean(yValidation),'Validation','HorizontalAlignment','center',...
        'VerticalAlignment','middle');
else
    gHandle.validation.Vertices=[xValidation',yValidation'];
    gHandle.validationText.Position = [mean(xValidation),mean(yValidation)];
end

% ValidationUsed
xValidationUsed = epoch-latency+[0,validation,validation,0];
yValidationUsed = [2*bar_width,2*bar_width,3*bar_width,3*bar_width];
if ~isfield(gHandle,'validationUsed')
    gHandle.validationUsed=patch(xValidationUsed, yValidationUsed, ...
        validationUsed_color);
    gHandle.validationUsedText = text(mean(xValidationUsed),mean(yValidationUsed),'Validation Used','HorizontalAlignment','center',...
        'VerticalAlignment','middle');
else
    gHandle.validationUsed.Vertices=[xValidationUsed',yValidationUsed'];
    gHandle.validationUsedText.Position = [mean(xValidationUsed),mean(yValidationUsed)];
end

% if ~isfield(gHandle,'trainText')
%     gHandle.trainText = text(mean(xTraining),mean(yTraining),'Training','HorizontalAlignment','center',...
%         'VerticalAlignment','middle');
% end

% Set the xTick
%
latencyDay = epoch_initial - latency_initial;
leadDay = epoch_initial + lead_initial;
trainingMin = epoch_initial - latency_initial - training_initial;
trainingUsedMin  = epoch_initial - 2*latency_initial - lead_initial ...
    - training_initial;
trainingUsedMax  = epoch_initial - 2*latency_initial - lead_initial;
validationUsedMax = epoch_initial - latency_initial + ...
    validation_initial;
validationMax = epoch_initial + lead_initial + validation_initial;

format = 'mm/dd/yy';
tickValue = [min(xTraining);min(xTrainingUsed);max(xTraining);max(xTrainingUsed);...
    epoch;max(xLead);max(xValidationUsed);max(xValidation)];
tickLabel = {datestr(trainingMin,format);datestr(trainingUsedMin,format);...
    datestr(latencyDay,format);datestr(trainingUsedMax,format);...
    datestr(epoch_initial,format);datestr(leadDay,format);...
    datestr(validationUsedMax,format);datestr(validationMax,format)};

% Sort the tickValue in ascending order.
tblA = table(tickValue,tickLabel);
tblB = sortrows(tblA,'tickValue');
tickValue = tblB.tickValue.';
tickLabel = tblB.tickLabel.';
xticks(tickValue);
xticklabels(tickLabel);

% Set the tick to vertical and hide the y-axes
xtickangle(90);
set(gca,'Fontsize',8);
set(gca,'YTickLabel',[]);
box on
grid on
return
end
